--
--  AutoBar
--
--  Config functions
-- 
--  Maintained by Azethoth / Toadkiller of Proudmoore.  Original author Saien of Hyjal
--  http://www.curse-gaming.com/en/wow/addons-4430-1-autobar-toadkiller.html
--

AutoBarConfig = {};
local dewdrop = DewdropLib:GetInstance("1.0");


StaticPopupDialogs["AUTOBAR_ITEMENTRY"] = {
	text = AUTOBAR_CONFIG_CUSTOM_ENTRY,
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1, 
	maxLetters = 50,
	hasWideEditBox = 1,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."WideEditBox");
		local text = editBox:GetText();
		if (tonumber(text) and tonumber(text) > 0) then
			text = tonumber(text);
		end
		local tmp = AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting];
		if (type(tmp) == "table") then
			AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting][AutoBarConfigFrame_ConfigButton_ChooseCategory.editting] = text;
		else
			AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting] = text;
		end
		AutoBarConfigFrame_ConfigButton_ChooseCategory:Hide();
	end,
	OnShow = function()
		local editBox = getglobal(this:GetName().."WideEditBox");
		local tmp = AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting];
		local txt;
		if (type(tmp) == "table") then
			txt = AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting][AutoBarConfigFrame_ConfigButton_ChooseCategory.editting];
		else
			txt = AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting];
		end
		if (txt) then
			editBox:SetText(txt);
		else
			editBox:SetText("");
		end
		editBox:SetFocus();
	end,
	OnHide = function()
		local editBox = getglobal(this:GetName().."WideEditBox");
		if (ChatFrameEditBox:IsVisible()) then 
			ChatFrameEditBox:SetFocus();
		end
		editBox:SetText("");
	end,
	OnCancel = function()
		this:GetParent():Hide();
	end,
	EditBoxOnEnterPressed = function ()
		StaticPopupDialogs["AUTOBAR_ITEMENTRY"].OnAccept();
		this:GetParent():Hide();
	end,
	EditBoxOnEscapePressed = function ()
		this:GetParent():Hide();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};


function AutoBarConfig_Toggle(msg)
	if (AutoBarConfigFrame:IsVisible()) then
		AutoBarConfigFrame:Hide();
	else
		AutoBarConfigFrame:Show();
	end
end


function AutoBarConfig.OnLoad()
	SLASH_AUTOBAR1 = "/autobar";
	SLASH_AUTOBAR2 = "/atb";
	SlashCmdList["AUTOBAR"] = function (msg)
	    -- no damn keyboard commands, just toggle config dialog
		AutoBarConfig_Toggle(msg);
	end
end


function AutoBarConfig.OnShow()
	local index, button, hotkey;
	for index = 1, AUTOBAR_MAXBUTTONS, 1 do
		button = getglobal("AutoBarConfigFrame_Buttons_Button"..index);
		hotkey = getglobal("AutoBarConfigFrame_Buttons_Button"..index.."HotKey");
		count  = getglobal("AutoBarConfigFrame_Buttons_Button"..index.."Count");
		icon   = getglobal("AutoBarConfigFrame_Buttons_Button"..index.."Icon");
		hotkey:SetText("#"..index);
		if (AutoBar_Config[AutoBar_Player].buttons[index]) then
			local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[index];
			icon:SetTexture(AutoBar_GetTexture(buttonInfo));
			count:SetText("");
		else
			count:SetText(AUTOBAR_CONFIG_EMPTY);
			icon:SetTexture("");
		end
	end
	AutoBarConfigFrameSliders();
	AutoBarConfig:CheckBoxSetup();
	AutoBarConfigSlot:Setup();
	AutoBarConfig:DockToInitialize();
end


function AutoBar_Config_ButtonSetTooltip()
	local buttonInfo;
	local preamble, extended;
	if (this.itemid) then
		local name,itemid = GetItemInfo(this.itemid);
		if (name and itemid) then
			GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
			GameTooltip:SetHyperlink(itemid);
		else
			local tmp = "item:"..this.itemid..":0:0:0";
			GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
			GameTooltip:SetHyperlink("item:"..this.itemid..":0:0:0");
		end
		return;
	end
	if (this.category) then
		buttonInfo = this.category;
		extended=true;
	elseif (string.find(this:GetName(), "^AutoBarConfigFrame_Buttons_Button")) then
		buttonInfo = AutoBar_Config[AutoBar_Player].buttons[this:GetID()];
		extended=true;
		preamble=true;
	elseif (string.find(this:GetName(), "^AutoBarConfigFrame_ConfigButton_Button")) then
		extended=true;
		local tmp = AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting];
		if (type(tmp) == "table") then
			buttonInfo = tmp[this:GetID()]
		else
			buttonInfo = tmp;
		end
	end
	local message = "";
	if (buttonInfo == "EMPTY") then
	elseif (type(buttonInfo) == "table") then
		if (preamble) then message = AUTOBAR_TOOLTIP9; end
		local index,cat;
		for index,cat in buttonInfo do
			if (type(cat) == "string" and AutoBar_Category_Info[cat]) then
				message = message.."\n"..AutoBar_Category_Info[cat].description;
			elseif (type(cat) == "number") then
				local name = GetItemInfo(cat);
				if (name) then
					message = message.."\n"..name..AUTOBAR_TOOLTIP10;
				else
					message = message..AUTOBAR_TOOLTIP11;
				end
--			else
--				message = message.."\n"..cat..AUTOBAR_TOOLTIP12;
			end
		end
	elseif (type(buttonInfo) == "string" and AutoBar_Category_Info[buttonInfo]) then
		if (preamble) then message = AUTOBAR_TOOLTIP13; end
		message = message..AutoBar_Category_Info[buttonInfo].description;
		if (extended) then
			message = message.."\n";
			if (AutoBar_Category_Info[buttonInfo].notusable) then
				message = message..AUTOBAR_TOOLTIP14;
			end
			if (AutoBar_Category_Info[buttonInfo].targetted) then
				if (AutoBar_Category_Info[buttonInfo].targetted == "WEAPON") then
					message = message..AUTOBAR_TOOLTIP15;
				else
					message = message..AUTOBAR_TOOLTIP16;
				end
			end
			if (AutoBar_Category_Info[buttonInfo].noncombat) then
				message = message..AUTOBAR_TOOLTIP17;
			end
			if (AutoBar_Category_Info[buttonInfo].combatonly) then
				message = message..AUTOBAR_TOOLTIP18;
			end
			if (AutoBar_Category_Info[buttonInfo].location) then
				message = message..AUTOBAR_TOOLTIP19..AutoBar_Category_Info[buttonInfo].location..".";
			end
			if (AutoBar_Category_Info[buttonInfo].limit) then
				message = message..AUTOBAR_TOOLTIP20;
				if (AutoBar_Category_Info[buttonInfo].limit.downhp) then
					message = message..AUTOBAR_TOOLTIP21;
					if (AutoBar_Category_Info[buttonInfo].limit.downmana) then
						message = message..", ";
					end
				end
				if (AutoBar_Category_Info[buttonInfo].limit.downmana) then
					message = message..AUTOBAR_TOOLTIP22;
				end
			end

		end
	elseif (type(buttonInfo) == "string" and not AutoBar_Category_Info[buttonInfo]) then
		if (preamble) then message = AUTOBAR_TOOLTIP23; end
		message = message..buttonInfo..AUTOBAR_TOOLTIP12;
	elseif (type(buttonInfo) == "number") then
		if (preamble) then message = AUTOBAR_TOOLTIP23; end
		local name = GetItemInfo(buttonInfo);
		if (name) then
			message = message..name..AUTOBAR_TOOLTIP10;
		else
			message = message..AUTOBAR_TOOLTIP11;
		end
	end

	GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
	GameTooltip:SetText(message);
end


function AutoBar_Config_Button_Edit(slotIndex)
	if (not slotIndex) then slotIndex = AutoBarConfigFrame_ConfigButton.editting; end
	local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[slotIndex];
	AutoBarConfigFrame_ConfigButton.editting = slotIndex;
	AutoBarConfigFrame_ConfigButton:Show();
	AutoBarConfigFrame_ConfigButtonTitleText:SetText(AUTOBAR_CONFIG_EDITBUTTON.." #"..slotIndex);
	local noPopupCheckbox = getglobal("AutoBarConfigFrame_ConfigButton_NoPopup");
	local arrangeOnUseCheckbox = getglobal("AutoBarConfigFrame_ConfigButton_ArrangeOnUse");
	local rightClickTargetsPetCheckbox = getglobal("AutoBarConfigFrame_ConfigButton_RightClickTargetsPet");
	local noPopupCheckboxText = getglobal("AutoBarConfigFrame_ConfigButton_NoPopupText");
	local arrangeOnUseCheckboxText = getglobal("AutoBarConfigFrame_ConfigButton_ArrangeOnUseText");
	local rightClickTargetsPetCheckboxText = getglobal("AutoBarConfigFrame_ConfigButton_RightClickTargetsPetText");
	local index,tmp,i;

	if (type(buttonInfo) == "table") then
		noPopupCheckbox:Enable();
		arrangeOnUseCheckbox:Enable();
		rightClickTargetsPetCheckbox:Enable();

		noPopupCheckboxText:SetVertexColor(NORMAL_FONT_COLOR.r , NORMAL_FONT_COLOR.g , NORMAL_FONT_COLOR.b);
		arrangeOnUseCheckboxText:SetVertexColor(NORMAL_FONT_COLOR.r , NORMAL_FONT_COLOR.g , NORMAL_FONT_COLOR.b);
		rightClickTargetsPetCheckboxText:SetVertexColor(NORMAL_FONT_COLOR.r , NORMAL_FONT_COLOR.g , NORMAL_FONT_COLOR.b);

		noPopupCheckbox:SetChecked(buttonInfo.noPopup);
		arrangeOnUseCheckbox:SetChecked(buttonInfo.arrangeOnUse);
		rightClickTargetsPetCheckbox:SetChecked(buttonInfo.rightClickTargetsPet);

		tmp = 0;
		for index = 1, AUTOBAR_MAXSLOTCATEGORIES, 1 do
			if (AutoBar_Config[AutoBar_Player].buttons[slotIndex][index]) then
				tmp = index;
			end
		end
		index = 1;
		while (index < tmp) do 
			if (AutoBar_Config[AutoBar_Player].buttons[slotIndex][index]) then
				index = index + 1;
			else
				AutoBar_Config[AutoBar_Player].buttons[slotIndex][index] = 
				AutoBar_Config[AutoBar_Player].buttons[slotIndex][index+1];
				AutoBar_Config[AutoBar_Player].buttons[slotIndex][index+1] = nil;
				tmp = 0;
				for i = 1, AUTOBAR_MAXSLOTCATEGORIES, 1 do
					if (AutoBar_Config[AutoBar_Player].buttons[slotIndex][i]) then
						tmp = i;
					end
				end
			end
		end
	else
		noPopupCheckbox:Disable();
		arrangeOnUseCheckbox:Disable();
		rightClickTargetsPetCheckbox:Disable();

		noPopupCheckboxText:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		arrangeOnUseCheckboxText:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		rightClickTargetsPetCheckboxText:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
	end

	for index = 1, AUTOBAR_MAXSLOTCATEGORIES, 1 do
		local button   = getglobal("AutoBarConfigFrame_ConfigButton_Button"..index);
		local hotkey   = getglobal("AutoBarConfigFrame_ConfigButton_Button"..index.."HotKey");
		local count    = getglobal("AutoBarConfigFrame_ConfigButton_Button"..index.."Count");
		local icon     = getglobal("AutoBarConfigFrame_ConfigButton_Button"..index.."Icon");
		local move     = getglobal("AutoBarConfigFrame_ConfigButton_Move"..index);
		local moveup   = getglobal("AutoBarConfigFrame_ConfigButton_Move"..index.."_Up");
		local movedown = getglobal("AutoBarConfigFrame_ConfigButton_Move"..index.."_Down");
		local checkbox = getglobal("AutoBarConfigFrame_ConfigButton_Option"..index);
		local checkboxtext = getglobal("AutoBarConfigFrame_ConfigButton_Option"..index.."Text");

		hotkey:Hide();
		if (type(buttonInfo) == "table") then
			if (buttonInfo[index]) then
				move:Show();
				count:SetText("");
				if (AutoBar_Category_Info[buttonInfo[index]] and AutoBar_Category_Info[buttonInfo[index]].targetted) then
					checkbox:Show();
					checkboxtext:SetText(AUTOBAR_CONFIG_SMARTSELFCAST);
					if (AutoBar_Config[AutoBar_Player].smartselfcast and AutoBar_Config[AutoBar_Player].smartselfcast[buttonInfo[index]]) then
						checkbox:SetChecked(1);
					else
						checkbox:SetChecked(0);
					end
				else
					checkbox:Hide();
				end
				if (index == 1) then
					moveup:Hide();
				else
					moveup:Show();
				end
				if (buttonInfo[index+1]) then
					movedown:Show();
				else
					movedown:Hide();
				end
			else
				move:Hide();
				count:SetText("Empty");
				checkbox:Hide();
			end
			icon:SetTexture(AutoBar_GetTexture(buttonInfo[index]));
			button:Show();
		else
			move:Hide();
			if (index == 1) then
				icon:SetTexture(AutoBar_GetTexture(buttonInfo));
				button:Show();
				if (buttonInfo) then
					count:SetText("");
					if (AutoBar_Category_Info[buttonInfo] and AutoBar_Category_Info[buttonInfo].targetted) then
						checkbox:Show();
						checkboxtext:SetText(AUTOBAR_CONFIG_SMARTSELFCAST);
						if (AutoBar_Config[AutoBar_Player].smartselfcast and AutoBar_Config[AutoBar_Player].smartselfcast[buttonInfo]) then
							checkbox:SetChecked(1);
						else
							checkbox:SetChecked(0);
						end
					else
						checkbox:Hide();
					end
				else
					count:SetText(AUTOBAR_CONFIG_EMPTY);
					checkbox:Hide();
				end
			else
				button:Hide();
				checkbox:Hide();
			end
		end
	end
	---
	if (type(buttonInfo) == "table") then
		AutoBarConfigFrame_ConfigButton_ConvertButton:SetText(AUTOBAR_CONFIG_CONVERTBUTTON1);
	else
		AutoBarConfigFrame_ConfigButton_ConvertButton:SetText(AUTOBAR_CONFIG_CONVERTBUTTON2);
	end
end


-- Toggle between single category slot & multi category slot.
function AutoBarConfig:ConvertSlotButtonOnClick()
	local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting];
	local noPopupCheckbox = getglobal("AutoBarConfigFrame_ConfigButton_NoPopup");
	local arrangeOnUseCheckbox = getglobal("AutoBarConfigFrame_ConfigButton_ArrangeOnUse");
	local rightClickTargetsPetCheckbox = getglobal("AutoBarConfigFrame_ConfigButton_RightClickTargetsPet");
	local noPopupCheckboxText = getglobal("AutoBarConfigFrame_ConfigButton_NoPopupText");
	local arrangeOnUseCheckboxText = getglobal("AutoBarConfigFrame_ConfigButton_ArrangeOnUseText");
	local rightClickTargetsPetCheckboxText = getglobal("AutoBarConfigFrame_ConfigButton_RightClickTargetsPetText");

	if (type(buttonInfo) == "table") then
		local tmp = buttonInfo[1];
		AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting] = tmp;

		noPopupCheckbox:Disable();
		arrangeOnUseCheckbox:Disable();
		rightClickTargetsPetCheckbox:Disable();

		noPopupCheckboxText:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		arrangeOnUseCheckboxText:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		arrangeOnUseCheckboxText:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
	else
		local tmp = buttonInfo;
		AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting] = { tmp };
		noPopupCheckbox:Enable();
		arrangeOnUseCheckbox:Enable();
		rightClickTargetsPetCheckbox:Enable();

		noPopupCheckboxText:SetVertexColor(NORMAL_FONT_COLOR.r , NORMAL_FONT_COLOR.g , NORMAL_FONT_COLOR.b);
		arrangeOnUseCheckboxText:SetVertexColor(NORMAL_FONT_COLOR.r , NORMAL_FONT_COLOR.g , NORMAL_FONT_COLOR.b);
		rightClickTargetsPetCheckboxText:SetVertexColor(NORMAL_FONT_COLOR.r , NORMAL_FONT_COLOR.g , NORMAL_FONT_COLOR.b);
	end
	AutoBar_Config_Button_Edit(AutoBarConfigFrame_ConfigButton.editting);
end


-- Delete all items in a slot.
function AutoBarConfig:ClearSlotButtonOnClick()
	local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting];
	if (type(buttonInfo) == "table") then
		local tmp = buttonInfo[1];
		AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting] = {};
	else
		local tmp = buttonInfo;
		AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting] = nil;
	end
	AutoBar_Config_Button_Edit(AutoBarConfigFrame_ConfigButton.editting);
end


-- Hook PickupContainerItem so we know what bag & slot is being dragged from
AutoBarConfig.PickupContainerItem = PickupContainerItem;
local dragBag = nil;
local dragSlot = nil;

function PickupContainerItem(bag, slot, ...)
	local returnTab = { AutoBarConfig.PickupContainerItem(bag, slot, unpack(arg)) };
	
	if (CursorHasItem()) then
		dragBag = bag;
		dragSlot = slot;
	end
	
	return unpack(returnTab);
end


local function AutoBar_ConfigButton_OnClick(mousebutton)
	if (CursorHasItem() and dragBag and dragSlot) then
        AutoBarConfig.ButtonOnReceiveDrag();
	else
		local tmp = AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting];
		if (type(tmp) == "table") then
			AutoBarConfigFrame_ConfigButton_ChooseCategory.editting = this:GetID();
		else
			AutoBarConfigFrame_ConfigButton_ChooseCategory.editting = nil;
		end
		AutoBarConfigFrame_ConfigButton_ChooseCategory:Show();
	end
end


local function AutoBar_ConfigButton_ChooseCategory_OnClick(mousebutton)
	if (IsShiftKeyDown()) then
		if (AutoBarConfigFrame_ConfigButton_ChooseCategory.categoryexplore) then
			AutoBarConfigFrame_ConfigButton_ChooseCategory.categoryexplore = nil;
			this:GetParent():Hide();
		else
			local category = this.category;
			if (category == "EMPTY") then category = nil; end
			if (category) then
				AutoBarConfigFrame_ConfigButton_ChooseCategory.categoryexplore = category;
				AutoBar_Config_ButtonChooseCategory_OnShow();
			end
		end
	else
		local category = this.category;
		if (category == "EMPTY") then category = nil; end
		if (AutoBarConfigFrame_ConfigButton_ChooseCategory.editting) then
			AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting][AutoBarConfigFrame_ConfigButton_ChooseCategory.editting] = category;
		else
			AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting] = category;
		end
		AutoBarConfigFrame_ConfigButton_ChooseCategory.categoryexplore = nil;
		this:GetParent():Hide();
	end
end
	

function AutoBar_Config_Button_OnClick(mousebutton)
	ResetCursor();
	AutoBarConfigFrame_ConfigButton.dragging = nil;
	AutoBarConfigFrame_ConfigButton.draggingConfigButton = nil;
	if (string.find(this:GetName(), "^AutoBarConfigFrame_Buttons_Button")) then
		return AutoBar_Config_Button_Edit(this:GetID());
	elseif (string.find(this:GetName(), "^AutoBarConfigFrame_ConfigButton_Button")) then
		return AutoBar_ConfigButton_OnClick(mousebutton);
	elseif (string.find(this:GetName(), "^AutoBarConfigFrame_ConfigButton_ChooseCategory_Button")) then
		return AutoBar_ConfigButton_ChooseCategory_OnClick(mousebutton);
	end
end


function AutoBarConfig.ButtonOnDragStart()
	local fromIndex = this:GetID();
	if (string.find(this:GetName(), "^AutoBarConfigFrame_Buttons_Button")) then
		AutoBarConfigFrame_ConfigButton.dragging = fromIndex;
	elseif (string.find(this:GetName(), "^AutoBarConfigFrame_ConfigButton_Button")) then
		AutoBarConfigFrame_ConfigButton.draggingConfigButton = fromIndex;
	end
	SetCursor("BUY_CURSOR");
end


function AutoBarConfig.ButtonOnReceiveDrag()
	local toIndex = this:GetID();
	if (string.find(this:GetName(), "^AutoBarConfigFrame_Buttons_Button")) then
		local fromIndex = AutoBarConfigFrame_ConfigButton.dragging;
		if (fromIndex and fromIndex ~= toIndex) then
			local fromButton = AutoBar_Config[AutoBar_Player].buttons[fromIndex];
			AutoBar_Config[AutoBar_Player].buttons[fromIndex] = AutoBar_Config[AutoBar_Player].buttons[toIndex];
			AutoBar_Config[AutoBar_Player].buttons[toIndex] = fromButton;
		end
	elseif (string.find(this:GetName(), "^AutoBarConfigFrame_ConfigButton_Button")) then
		local fromIndex = AutoBarConfigFrame_ConfigButton.draggingConfigButton;
--		if (fromIndex) then
--			DEFAULT_CHAT_FRAME:AddMessage("AutoBarConfig.ButtonOnReceiveDrag " .. fromIndex .. " -> ", 1, 0.5, 0);
--		end
--		DEFAULT_CHAT_FRAME:AddMessage("AutoBarConfig.ButtonOnReceiveDrag -> " .. toIndex, 1, 0.5, 0);
		if (fromIndex and fromIndex ~= toIndex) then
			AutoBar_MoveButtonItems(AutoBarConfigFrame_ConfigButton.editting, fromIndex, toIndex);
		elseif (CursorHasItem()) then
			if (dragBag and dragSlot) then
				local button = AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting];
				local name, id = AutoBar.LinkDecode(GetContainerItemLink(dragBag, dragSlot));
				if (type(button) == "table") then
					if (table.getn(button) < toIndex) then
						toIndex = table.getn(button) + 1;	
					end
					AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting][toIndex] = id;
				else
					AutoBar_Config[AutoBar_Player].buttons[AutoBarConfigFrame_ConfigButton.editting] = id;
				end
				dragBag = nil;
				dragSlot = nil;
--				PutItemInBackpack();
				ClearCursor();
			end
		end                                                          
		AutoBar_Config_Button_Edit(AutoBarConfigFrame_ConfigButton.editting);
	end
	AutoBarConfig.OnShow();
	AutoBar.ConfigUpdated();
	AutoBarConfigFrame_ConfigButton.dragging = nil;
	AutoBarConfigFrame_ConfigButton.draggingConfigButton = nil;
	ResetCursor();
end


function AutoBarConfig.OnEnter()
	if (AutoBarConfigFrame_ConfigButton.dragging) then
		SetCursor("BUY_CURSOR");
	end
end


function AutoBar_Config_ButtonChooseCategory_OnShow()
	if (AutoBarConfigFrame_ConfigButton_ChooseCategory.categoryexplore and not AutoBar_Category_Info[AutoBarConfigFrame_ConfigButton_ChooseCategory.categoryexplore]) then
		AutoBarConfigFrame_ConfigButton_ChooseCategory.categoryexplore = nil;
	end
	if (AutoBarConfigFrame_ConfigButton_ChooseCategory.categoryexplore) then
		AutoBarConfigFrame_ConfigButton_ChooseCategory_HintText1:Hide();
		local category = AutoBarConfigFrame_ConfigButton_ChooseCategory.categoryexplore;

		FauxScrollFrame_Update(AutoBarConfigFrame_ConfigButton_ChooseCategory_Scroll, table.getn(AutoBar_Category_Info[category].items), 7, 36);
		local offset = FauxScrollFrame_GetOffset(AutoBarConfigFrame_ConfigButton_ChooseCategory_Scroll);
		local index,name,texture;
		for index = 1, 7, 1 do
			local button = getglobal("AutoBarConfigFrame_ConfigButton_ChooseCategory_Button"..index);
			local hotkey = getglobal("AutoBarConfigFrame_ConfigButton_ChooseCategory_Button"..index.."HotKey");
			local count  = getglobal("AutoBarConfigFrame_ConfigButton_ChooseCategory_Button"..index.."Count");
			local icon   = getglobal("AutoBarConfigFrame_ConfigButton_ChooseCategory_Button"..index.."Icon");
			local text   = getglobal("AutoBarConfigFrame_ConfigButton_ChooseCategory_Text"..index);
			button.category = nil;
			if (AutoBar_Category_Info[category].items[index+offset]) then
				if (type(AutoBar_Category_Info[category].items[index+offset]) == "number") then
					name,_,_,_,_,_,_,_,texture = GetItemInfo(AutoBar_Category_Info[category].items[index+offset]);
					if (not name) then
						name = AUTOBAR_CONFIG_NOTFOUND..AutoBar_Category_Info[category].items[index+offset]..")";
						texture = "Interface\\Icons\\INV_Misc_Gift_01";
					elseif (not texture) then
						texture = "Interface\\Icons\\INV_Misc_Gift_02";
					end
				else
					texture = "Interface\\Icons\\INV_Misc_Gift_03";
					name = AutoBar_Category_Info[category].items[index+offset];
				end
				icon:SetTexture(texture);
				text:SetText(name);
				count:SetText("");
				button:Show();
				button.itemid = tonumber(AutoBar_Category_Info[category].items[index+offset]);
			else
				button:Hide();
				button.itemid = nil;
				text:SetText("");
			end
		end
	else
		AutoBarConfigFrame_ConfigButton_ChooseCategory_HintText1:Show();
		local sortedCategories = {};
		for categoryName, rec in AutoBar_Category_Info do
			table.insert(sortedCategories, categoryName);
		end
		table.sort (sortedCategories);
		table.insert(sortedCategories, 1, "EMPTY");
		FauxScrollFrame_Update(AutoBarConfigFrame_ConfigButton_ChooseCategory_Scroll, table.getn(sortedCategories), 7, 36);
		local offset = FauxScrollFrame_GetOffset(AutoBarConfigFrame_ConfigButton_ChooseCategory_Scroll);
		local index;
		for index = 1, 7, 1 do
			local button = getglobal("AutoBarConfigFrame_ConfigButton_ChooseCategory_Button"..index);
			local hotkey = getglobal("AutoBarConfigFrame_ConfigButton_ChooseCategory_Button"..index.."HotKey");
			local count  = getglobal("AutoBarConfigFrame_ConfigButton_ChooseCategory_Button"..index.."Count");
			local icon   = getglobal("AutoBarConfigFrame_ConfigButton_ChooseCategory_Button"..index.."Icon");
			local text   = getglobal("AutoBarConfigFrame_ConfigButton_ChooseCategory_Text"..index);
			button.itemid = nil;
			if (sortedCategories[index+offset] == "EMPTY") then
				icon:SetTexture("");
				count:SetText(AUTOBAR_CONFIG_EMPTY);
				text:SetText(AUTOBAR_CONFIG_REMOVECAT);
				button:Show();
				button.category = sortedCategories[index+offset];
			elseif (sortedCategories[index+offset]) then
				icon:SetTexture(AutoBar_GetTexture(sortedCategories[index+offset]));
				count:SetText("");
				text:SetText(AutoBar_Category_Info[sortedCategories[index+offset]].description);
				button.category = sortedCategories[index+offset];
				button:Show();
			else
				button:Hide();
				button.category = nil;
				text:SetText("");
			end
		end
	end
end

function AutoBar_Config_ButtonChooseCategory_OnScroll()
	GameTooltip:Hide();
	AutoBar_Config_ButtonChooseCategory_OnShow();
end

function AutoBar_Config_Button_InsertCustomItem()
	StaticPopup_Show("AUTOBAR_ITEMENTRY");
end


local function AutoBarConfigFrameSliderGetValue(sliderName, calledFromSlider)
	local value = nil;
	if (calledFromSlider) then
		value = getglobal(AutoBarConfigSliders[sliderName]["name"]):GetValue();
	end
	if ((not value or value == 0) and AutoBar_Config[AutoBar_Player].display[sliderName]) then 
		value = AutoBar_Config[AutoBar_Player].display[sliderName];
	elseif (not value or value == 0) then 
		value = AutoBarConfigSliders[sliderName]["default"]; 
	end
	return value;
end


local function AutoBarConfigFrameSliderSetValue(sliderName, value, textValue)
	if (not textValue) then
		textValue = tostring(value);
	end
--DEFAULT_CHAT_FRAME:AddMessage("AutoBarConfigFrameSliderSetValue " ..sliderName);
	getglobal(AutoBarConfigSliders[sliderName]["name"] .. "Text"):SetText(AutoBarConfigSliders[sliderName]["text"]..textValue);
	getglobal(AutoBarConfigSliders[sliderName]["name"]):SetValue(value);
end


function AutoBarConfigFrameSliders(calledFromSlider)
	if (not AutoBar_Config[AutoBar_Player].display) then
		AutoBar_Config[AutoBar_Player].display = {}; 
	end

	local rows = AutoBarConfigFrameSliderGetValue("rows", calledFromSlider);
	local columns = AutoBarConfigFrameSliderGetValue("columns", calledFromSlider);
	local gapping = AutoBarConfigFrameSliderGetValue("gapping", calledFromSlider);
	local alpha = AutoBarConfigFrameSliderGetValue("alpha", calledFromSlider);
	local buttonWidth = AutoBarConfigFrameSliderGetValue("buttonWidth", calledFromSlider);
	local buttonHeight = AutoBarConfigFrameSliderGetValue("buttonHeight", calledFromSlider);
	local dockShiftX = AutoBarConfigFrameSliderGetValue("dockShiftX", calledFromSlider);
	local dockShiftY = AutoBarConfigFrameSliderGetValue("dockShiftY", calledFromSlider);

	if (calledFromSlider) then
		if (rows * columns > AUTOBAR_MAXBUTTONS) then
			if (this:GetName() == AutoBarConfigSliders["rows"]["name"]) then
				rows = getglobal(AutoBarConfigSliders["rows"]["name"]):GetValue();
				columns = math.floor(AUTOBAR_MAXBUTTONS / rows);
			else
				columns = getglobal(AutoBarConfigSliders["columns"]["name"]):GetValue();
				rows = math.floor(AUTOBAR_MAXBUTTONS / columns);
			end
		end
		if (buttonHeight ~= buttonWidth and not AutoBar_Config[AutoBar_Player].display.widthHeightUnlocked) then
			if (this:GetName() == AutoBarConfigSliders["buttonWidth"]["name"]) then
				buttonHeight = buttonWidth;
			elseif (this:GetName() == AutoBarConfigSliders["buttonHeight"]["name"]) then
				buttonWidth = buttonHeight;
			end
		end

		AutoBar_Config[AutoBar_Player].display.rows = rows;
		AutoBar_Config[AutoBar_Player].display.columns = columns;
		AutoBar_Config[AutoBar_Player].display.gapping = gapping;
		AutoBar_Config[AutoBar_Player].display.alpha = alpha;
		AutoBar_Config[AutoBar_Player].display.buttonWidth = buttonWidth;
		AutoBar_Config[AutoBar_Player].display.buttonHeight = buttonHeight;
		AutoBar_Config[AutoBar_Player].display.dockShiftX = dockShiftX;
		AutoBar_Config[AutoBar_Player].display.dockShiftY = dockShiftY;
	end

    AutoBarConfigFrameSliderSetValue("rows", rows);
    AutoBarConfigFrameSliderSetValue("columns", columns);
    AutoBarConfigFrameSliderSetValue("gapping", gapping);
    AutoBarConfigFrameSliderSetValue("alpha", alpha, math.floor(alpha) / 10);
    AutoBarConfigFrameSliderSetValue("buttonWidth", buttonWidth);
    AutoBarConfigFrameSliderSetValue("buttonHeight", buttonHeight);
    AutoBarConfigFrameSliderSetValue("dockShiftX", dockShiftX);
    AutoBarConfigFrameSliderSetValue("dockShiftY", dockShiftY);

	AutoBar_SetupVisual();
end


AutoBarConfigCheckboxes =
{
	["reverseButtons"] =
	{
		["name"] = "AutoBarConfigFrameBarReverseButtons",
		["text"] = AUTOBAR_CONFIG_REVERSE,
	},
	["autoSmartSelfCast"] =
	{
		["name"] = "AutoBarConfigFrameBarSmartSelfCast",
		["text"] = AUTOBAR_CONFIG_SMARTSELFCAST,
	},
	["showEmptyButtons"] =
	{
		["name"] = "AutoBarConfigFrameBarShowEmptyButtons",
		["text"] = AUTOBAR_CONFIG_SHOWEMPTY,
	},
	["showCategoryIcon"] =
	{
		["name"] = "AutoBarConfigFrameBarShowCategoryIcon",
		["text"] = AUTOBAR_CONFIG_SHOWCATEGORYICON,
	},
	["hideTooltips"] =
	{
		["name"] = "AutoBarConfigFrameBarHideTooltips",
		["text"] = AUTOBAR_CONFIG_HIDETOOLTIP,
	},
	["hideDragHandle"] =
	{
		["name"] = "AutoBarConfigFrameBarHideDragHandle",
		["text"] = AUTOBAR_CONFIG_HIDEDRAGHANDLE,
	},
	["widthHeightUnlocked"] =
	{
		["name"] = "AutoBarConfigFrameButtonsWidthHeightUnlocked",
		["text"] = AUTOBAR_CONFIG_WIDTHHEIGHTUNLOCKED,
	},
	["hideKeyText"] =
	{
		["name"] = "AutoBarConfigFrameButtonsHideKeyText",
		["text"] = AUTOBAR_CONFIG_HIDEKEYBINDING,
	},
	["hideCount"] =
	{
		["name"] = "AutoBarConfigFrameButtonsHideCount",
		["text"] = AUTOBAR_CONFIG_HIDECOUNT,
	},
	["plainButtons"] =
	{
		["name"] = "AutoBarConfigFrameButtonsPlainButtons",
		["text"] = AUTOBAR_CONFIG_PLAINBUTTONS,
	},
	["popupToTop"] =
	{
		["name"] = "AutoBarConfigFramePopupPopupToTop",
		["text"] = nil,
		["group"] = "PopupTo",
	},
	["popupToLeft"] =
	{
		["name"] = "AutoBarConfigFramePopupPopupToLeft",
		["text"] = nil,
		["group"] = "PopupTo",
	},
	["popupToRight"] =
	{
		["name"] = "AutoBarConfigFramePopupPopupToRight",
		["text"] = AUTOBAR_CONFIG_POPUPDIRECTION,
		["group"] = "PopupTo",
	},
	["popupToBottom"] =
	{
		["name"] = "AutoBarConfigFramePopupPopupToBottom",
		["text"] = nil,
		["group"] = "PopupTo",
	},
	["popupDisable"] =
	{
		["name"] = "AutoBarConfigFramePopupPopupDisable",
		["text"] = AUTOBAR_CONFIG_POPUPDISABLE,
	},
	["popupOnShift"] =
	{
		["name"] = "AutoBarConfigFramePopupPopupOnShift",
		["text"] = AUTOBAR_CONFIG_POPUPONSHIFT,
	},
}

function AutoBarConfig:CheckBoxSetup()
	for key, checkboxData in AutoBarConfigCheckboxes do
		local checkbox = getglobal(checkboxData.name);
		local checkboxText = getglobal(checkboxData.name.."Text");

--DEFAULT_CHAT_FRAME:AddMessage("AutoBarConfig:CheckBoxSetup " ..checkboxData.name.. " " .. checkboxData.name.."Text");
		checkboxText:SetText(checkboxData.text);
		if (not checkboxData.text) then
			checkboxText:Hide();
		end

        checkbox:SetChecked(AutoBar_Config[AutoBar_Player].display[key]);
	end
end


function AutoBarConfig:CheckBoxOnClick()
	local buttonName = this:GetName();
	local group = nil; 
	for key, checkboxData in AutoBarConfigCheckboxes do
		if (buttonName == checkboxData["name"]) then
			AutoBar_Config[AutoBar_Player].display[key] = this:GetChecked();
			
			-- Radio button behavior
			group = checkboxData["group"];
			if (group) then
				-- Set clicked one to true
				AutoBar_Config[AutoBar_Player].display[key] = true;
				this:SetChecked(true);
				
				-- Set other group members to false
				for keyGroup, checkboxDataGroup in AutoBarConfigCheckboxes do
					if (buttonName ~= checkboxDataGroup.name and group == checkboxDataGroup.group) then
						AutoBar_Config[AutoBar_Player].display[keyGroup] = false;
						getglobal(checkboxDataGroup.name):SetChecked(false);
					end
				end
			end
		end
	end

	AutoBarConfig:CheckBoxSetup();
	AutoBar_SetupVisual();
end


--
-- Dropdown menu for docking to frame options
--

AutoBarConfig.dockingFrames = {
		["NONE"] = AUTOBAR_CONFIG_DOCKTONONE,
		["MAINMENU"] = AUTOBAR_CONFIG_DOCKTOMAIN,
		["GSecondBar"] = AUTOBAR_CONFIG_DOCKTOGBARS,
	};

function AutoBarConfig:DockToInitialize()
	if (not AutoBarConfigFrameBarDockTo.dewdrop) then
		AutoBarConfigFrameBarDockToButton.dewdrop = dewdrop;
		dewdrop:Register(AutoBarConfigFrameBarDockTo, 'children', AutoBarConfig_DockToSetup,
			'point', function(parent) return "TOPLEFT", "TOPRIGHT" end
		);
	end
	local dockToFrame = AutoBarConfig:GetDockToFrame();
	AutoBarConfigFrameBarDockToText:SetText(AutoBarConfig.dockingFrames[dockToFrame]);
end


function AutoBarConfig:GetDockToFrame()
	local dockToFrame = "NONE";
	if (AutoBar_Config and AutoBar_Config[AutoBar_Player] and AutoBar_Config[AutoBar_Player].display and AutoBar_Config[AutoBar_Player].display.docking) then
		dockToFrame = AutoBar_Config[AutoBar_Player].display.docking;
	end
	return dockToFrame;
end


function AutoBarConfig_DockToSetup()
	local checked;
	local dockToFrame = AutoBarConfig:GetDockToFrame();
	for frameName, frameText in AutoBarConfig.dockingFrames do
--DEFAULT_CHAT_FRAME:AddMessage("AutoBarConfig_DockToSetup " ..frameName.. " " ..dockToFrame);
		if (frameName == dockToFrame) then
			checked = true;
		    AutoBarConfigFrameBarDockToText:SetText(frameText);
		else
			checked = false;
		end
		dewdrop:AddLine(
			'text', frameText,
			'checked', checked,
			'func', AutoBarConfig_DockToOnClick,
			'arg1', frameName,
			'arg2', frameText,
			'closeWhenClicked', true
		);
	end
end


-- DockTo Dropdown Menu callback
function AutoBarConfig_DockToOnClick(frameName, frameText)
	if (frameName ~= "NONE") then
		AutoBar_Config[AutoBar_Player].display.docking = frameName;
	else
		AutoBar_Config[AutoBar_Player].display.docking = nil;
	end
	--	dewdrop:Refresh();
    AutoBarConfigFrameBarDockToText:SetText(frameText);
	AutoBar_SetupVisual();
end


--
-- Config Checkbox Handling
--

function AutoBarConfig:CheckBoxOnCheck()
	local button = this:GetName(); 
	if (this:GetChecked()) then
		if (button == "AutoBarConfigFrame_Bar_SmartSelfCast") then
			AutoBar_Config[AutoBar_Player].autoSmartSelfCast = true;
		elseif (button == "AutoBarConfigFrame_Bar_WidthHeightLocked") then
			AutoBar_Config[AutoBar_Player].display.widthHeightUnlocked = nil;
		elseif (button == "AutoBarConfigFrame_Bar_ReverseButtons") then
			AutoBar_Config[AutoBar_Player].display.reversebuttons = 1;
		elseif (button == "AutoBarConfigFrame_Bar_HideKeyText") then
			AutoBar_Config[AutoBar_Player].display.hideKeyText = 1;
		elseif (button == "AutoBarConfigFrame_Bar_HideCount") then
			AutoBar_Config[AutoBar_Player].display.hideCount = 1;
		elseif (button == "AutoBarConfigFrame_Bar_ShowEmptyButtons") then
			AutoBar_Config[AutoBar_Player].display.showEmptyButtons = 1;
		elseif (button == "AutoBarConfigFrame_Bar_ShowCategoryIcon") then
			AutoBar_Config[AutoBar_Player].display.showCategoryIcon = 1;
		elseif (button == "AutoBarConfigFrame_Bar_HideTooltips") then
			AutoBar_Config[AutoBar_Player].display.hideTooltips = 1;
		elseif (button == "AutoBarConfigFrame_Bar_PopupToTop") then
			AutoBar_Config[AutoBar_Player].display.popupToTop = 1;
			AutoBar_Config[AutoBar_Player].display.popupToLeft = nil;
			AutoBar_Config[AutoBar_Player].display.popupToRight = nil;
			AutoBar_Config[AutoBar_Player].display.popupToBottom = nil;
		elseif (button == "AutoBarConfigFrame_Bar_PopupToLeft") then
			AutoBar_Config[AutoBar_Player].display.popupToTop = nil;
			AutoBar_Config[AutoBar_Player].display.popupToLeft = 1;
			AutoBar_Config[AutoBar_Player].display.popupToRight = nil;
			AutoBar_Config[AutoBar_Player].display.popupToBottom = nil;
		elseif (button == "AutoBarConfigFrame_Bar_PopupToRight") then
			AutoBar_Config[AutoBar_Player].display.popupToTop = nil;
			AutoBar_Config[AutoBar_Player].display.popupToLeft = nil;
			AutoBar_Config[AutoBar_Player].display.popupToRight = 1;
			AutoBar_Config[AutoBar_Player].display.popupToBottom = nil;
		elseif (button == "AutoBarConfigFrame_Bar_PopupToBottom") then
			AutoBar_Config[AutoBar_Player].display.popupToTop = nil;
			AutoBar_Config[AutoBar_Player].display.popupToLeft = nil;
			AutoBar_Config[AutoBar_Player].display.popupToRight = nil;
			AutoBar_Config[AutoBar_Player].display.popupToBottom = 1;
		elseif (button == "AutoBarConfigFrame_Bar_PopupDisable") then
			AutoBar_Config[AutoBar_Player].display.popupDisable = 1;
		elseif (button == "AutoBarConfigFrame_Bar_PopupOnShift") then
			AutoBar_Config[AutoBar_Player].display.popupOnShift = 1;
		elseif (button == "AutoBarConfigFrame_Bar_HideDragHandle") then
			AutoBar_Config[AutoBar_Player].display.hideDragHandle = 1;
		elseif (button == "AutoBarConfigFrame_Bar_PlainButtons") then
			AutoBar_Config[AutoBar_Player].display.plainButtons = 1;
		end
	else
		if (button == "AutoBarConfigFrame_Bar_SmartSelfCast") then
			AutoBar_Config[AutoBar_Player].autoSmartSelfCast = nil;
		elseif (button == "AutoBarConfigFrame_Bar_WidthHeightLocked") then
			AutoBar_Config[AutoBar_Player].display.widthHeightUnlocked = 1;
		elseif (button == "AutoBarConfigFrame_Bar_ReverseButtons") then
			AutoBar_Config[AutoBar_Player].display.reversebuttons = nil;
		elseif (button == "AutoBarConfigFrame_Bar_HideKeyText") then
			AutoBar_Config[AutoBar_Player].display.hideKeyText = nil;
		elseif (button == "AutoBarConfigFrame_Bar_HideCount") then
			AutoBar_Config[AutoBar_Player].display.hideCount = nil;
		elseif (button == "AutoBarConfigFrame_Bar_ShowEmptyButtons") then
			AutoBar_Config[AutoBar_Player].display.showEmptyButtons = nil;
		elseif (button == "AutoBarConfigFrame_Bar_ShowCategoryIcon") then
			AutoBar_Config[AutoBar_Player].display.showCategoryIcon = nil;
		elseif (button == "AutoBarConfigFrame_Bar_HideTooltips") then
			AutoBar_Config[AutoBar_Player].display.hideTooltips = nil;
		elseif (button == "AutoBarConfigFrame_Bar_PopupDisable") then
			AutoBar_Config[AutoBar_Player].display.popupDisable = nil;
		elseif (button == "AutoBarConfigFrame_Bar_PopupOnShift") then
			AutoBar_Config[AutoBar_Player].display.popupOnShift = nil;
		elseif (button == "AutoBarConfigFrame_Bar_HideDragHandle") then
			AutoBar_Config[AutoBar_Player].display.hideDragHandle = nil;
		elseif (button == "AutoBarConfigFrame_Bar_PlainButtons") then
			AutoBar_Config[AutoBar_Player].display.plainButtons = nil;
		end
	end
	AutoBarConfig:CheckBoxSetup();
	AutoBar_SetupVisual();
end


-- Individual smart self cast checkbox handling
function AutoBarConfigFrame_ConfigButton_Option_OnCheck()
	local slotIndex = AutoBarConfigFrame_ConfigButton.editting; 
	local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[slotIndex];
	local category;
	if (type(buttonInfo) == "table") then
		category = buttonInfo[this:GetID()];
	else
		category = buttonInfo;
	end
	if (not AutoBar_Config[AutoBar_Player].smartselfcast) then
		AutoBar_Config[AutoBar_Player].smartselfcast = {};
	end
	AutoBar_Config[AutoBar_Player].smartselfcast[category] = this:GetChecked();
	AutoBar_Config_Button_Edit();
end


AutoBarConfigSlot = {};
function AutoBarConfigSlot:Setup()
	getglobal("AutoBarConfigFrame_ConfigButton_NoPopupText"):SetText(AUTOBAR_CONFIG_NOPOPUP);
	getglobal("AutoBarConfigFrame_ConfigButton_ArrangeOnUseText"):SetText(AUTOBAR_CONFIG_ARRANGEONUSE);
	getglobal("AutoBarConfigFrame_ConfigButton_RightClickTargetsPetText"):SetText(AUTOBAR_CONFIG_RIGHTCLICKTARGETSPET);
end


function AutoBarConfigSlot:OnCheck()
	local slotIndex = AutoBarConfigFrame_ConfigButton.editting; 
	local buttonInfo = AutoBar_Config[AutoBar_Player].buttons[slotIndex];
	local buttonName = this:GetName(); 

	if (type(buttonInfo) == "table") then
		if (buttonName == "AutoBarConfigFrame_ConfigButton_NoPopup") then
			buttonInfo.noPopup = this:GetChecked();
		elseif (buttonName == "AutoBarConfigFrame_ConfigButton_ArrangeOnUse") then
			buttonInfo.arrangeOnUse = this:GetChecked();
		elseif (buttonName == "AutoBarConfigFrame_ConfigButton_RightClickTargetsPet") then
			buttonInfo.rightClickTargetsPet = this:GetChecked();
		end
	end
	AutoBar_Config_Button_Edit();
end


function AutoBarConfigFrame_MoveArrow_OnClick(fromIndex, direction)
	local button = AutoBarConfigFrame_ConfigButton.editting;
	if (type(AutoBar_Config[AutoBar_Player].buttons[button]) == "table") then
		local toIndex;
		if (direction == "UP") then
			toIndex = fromIndex - 1;
		elseif (direction == "DOWN") then
			toIndex = fromIndex + 1;
		end
		AutoBar_MoveButtonItems(button, fromIndex, toIndex);
		AutoBar_Config_Button_Edit(button);
	end
end


function AutoBar_MoveButtonItems(button, fromIndex, toIndex)
	if (type(AutoBar_Config[AutoBar_Player].buttons[button]) == "table") then
		if (not AutoBar_Config[AutoBar_Player].buttons[button][fromIndex] or fromIndex == toIndex) then
			-- Move to self so do nothing
			return;
		end
		if (not AutoBar_Config[AutoBar_Player].buttons[button][fromIndex]) then
			-- Dont allow swapping empties into the list
			return;
		end
		local temp = AutoBar_Config[AutoBar_Player].buttons[button][fromIndex];
		if (not AutoBar_Config[AutoBar_Player].buttons[button][toIndex]) then
			-- Move to the end
			table.remove(AutoBar_Config[AutoBar_Player].buttons[button], fromIndex);
			table.insert(AutoBar_Config[AutoBar_Player].buttons[button], temp);
		else
			-- Swap the two
			AutoBar_Config[AutoBar_Player].buttons[button][fromIndex] = AutoBar_Config[AutoBar_Player].buttons[button][toIndex];
			AutoBar_Config[AutoBar_Player].buttons[button][toIndex] = temp;
		end
	end
end


local tabFrameNames = { "Bar", "Buttons", "Popup", "Profile"  };

function AutoBarConfig:TabButtonOnClick(tabId)
	PanelTemplates_SetTab(AutoBarConfigFrame, tabId);
	PanelTemplates_UpdateTabs(AutoBarConfigFrame);
	AutoBar_Config[AutoBar_Player].display.selectedTab = tabId;
	for index, tabFrameName in tabFrameNames do
		if (tabFrameName == tabFrameNames[tabId]) then
			getglobal("AutoBarConfigFrame" .. tabFrameName):Show();
		else
			getglobal("AutoBarConfigFrame" .. tabFrameName):Hide();
		end
	end
end


-- Initialize the config tabs
function AutoBarConfig:TabButtonInitialize()
	PanelTemplates_SetNumTabs(AutoBarConfigFrame, 3);
	AutoBarConfigFrameTab4:Hide();
--	PanelTemplates_SetNumTabs(AutoBarConfigFrame, 4);
	if (not AutoBar_Config[AutoBar_Player].display.selectedTab) then
		AutoBar_Config[AutoBar_Player].display.selectedTab = 1;
	end

	AutoBarConfig:TabButtonOnClick(AutoBar_Config[AutoBar_Player].display.selectedTab)
	PanelTemplates_UpdateTabs(AutoBarConfigFrame);
end